/*  $Id: EThread.java 1335 2012-08-25 13:27:49Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.Iterator;
import java.util.Vector;

/**
 * Extended Thread.
 * makes thread naming mendatory.
 * Log trace of thread start request, thread starting, thread ending.
 */
public class EThread extends Thread {
    /**
     * Create new extended thread
     * @param name thread name.
     * @param run runnable to use as thread main proc.
     */ 
    public EThread(String name,Runnable run) {
        super(name);
        _toRun=run;
    }
    // Thread.start() override
    public void start() {
        App.get().log().debug(MSG_REQUEST+getId()+"["+getName()+"]");
        super.start();
    }
    // Thread.run() override
    public void run() {
    	synchronized (_activeThreads) {
    		_activeThreads.add(this);
    	}
        App.get().log().debug(MSG_START);
        try {
            _toRun.run();
            App.get().log().debug(MSG_COMPLETED);
        }
        catch (Exception ex) {
            App.get().log().warn(MSG_EXCEPTION+ex);
        }
    	synchronized (_activeThreads) {
    		_activeThreads.remove(this);
    	}
    }
    
    public boolean isCurrentThread() {
    	return Thread.currentThread()==this;
    }

	private static String objRef(Object obj) {
		return obj.getClass().getName()+"@"+Integer.toHexString(obj.hashCode());
	}
	public static void doWait(Object monitor) throws InterruptedException {
        App.get().log().debug(MSG_MONITOR_WAIT+objRef(monitor));
		monitor.wait();
        App.get().log().debug(MSG_MONITOR_CONT+objRef(monitor));
	}
	public static boolean doWait(Object monitor,long timeout) throws InterruptedException {
        App.get().log().debug(MSG_MONITOR_WAIT+objRef(monitor));
        // TODO check if java provide a more safe way to know if wait continues because of timeout or not.
        long start=System.currentTimeMillis();
		monitor.wait(timeout);
		long delay=System.currentTimeMillis()-start;
        App.get().log().debug(MSG_MONITOR_CONT+objRef(monitor));
        return delay<timeout;
	}
	public static void doWait(Object monitor,long timeout,int nanos) throws InterruptedException {
        App.get().log().debug(MSG_MONITOR_WAIT+objRef(monitor));
		monitor.wait(timeout,nanos);
        App.get().log().debug(MSG_MONITOR_CONT+objRef(monitor));
	}
	public static void doNotify(Object monitor) {
        App.get().log().debug(MSG_MONITOR_NOTIFY+objRef(monitor));
		monitor.notify();
	}
	public static void doNotifyAll(Object monitor) {
        App.get().log().debug(MSG_MONITOR_NOTIFY+objRef(monitor));
		monitor.notifyAll();
	}
    /** runnable to used as thread proc */
    private Runnable _toRun;
    
    /** */
    private static Vector<EThread> _activeThreads=new Vector<EThread>();
    public static Iterator<EThread> getActiveThreads() {
    	return _activeThreads.iterator();
    }

	public final static String MSG_REQUEST="Thread start request: ";
	public final static String MSG_START="Thread started.";
	public final static String MSG_COMPLETED="Thread completed.";
	public final static String MSG_ABORTED="Thread aborted.";
	public final static String MSG_EXCEPTION="Thread exception: ";
	public final static String MSG_SLEEP_START="Thread pause start: ";
	public final static String MSG_SLEEP_END="Thread pause end: ";
	public final static String MSG_MONITOR_WAIT="Monitor wait: ";
	public final static String MSG_MONITOR_CONT="Monitor continue: ";
	public final static String MSG_MONITOR_TIMEOUT="Monitor timeouted: ";
	public final static String MSG_MONITOR_NOTIFY="Monitor notify: ";
}
